#! /usr/bin/python
#
# h8trans.py
# (c) 2011 George Farris <farrisg@gmsys.com>	
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Version 0.1 - Feb 21, 2011 read and write CP/M disks
#         0.2 - Feb 22, 2011 added ability to read and write HDOS disks
#         0.3 - Feb 23, 2011 fixed set volume bug
#         0.4 - Feb 25, 2011 added config file support
#         0.5 - Feb 28, 2011 show cp/m disk image directory
#         0.6 - Mar 06, 2011 show HDOS disk image
#         0.7 - Nov 02, 2011 Fix oops if serial port is not in PORT_LIST
#			
#=============================================================================================


import sys, os, serial, string, binascii, time, ConfigParser
import gtk, pygtk, gobject, tempfile, getopt, datetime
#pygtk.require('2.0')

# ------------- User modifiable settings ------------------------------
# Default Serial port settings
SERIAL_PORT = '/dev/ttyUSB0'
BAUD_RATE = 9600
GLADEFILE="/usr/local/lib/h8trans/h8trans.glade"
PORT_LIST = ["/dev/ttyS0","/dev/ttyS1","/dev/ttyUSB0","/dev/ttyUSB1"]
BAUDRATE_LIST = [2400,4800,9600,19200,38400,57600]
# ------------- End of user modifiable settings -----------------------

INTERLEAVE_LIST = ["1:1","1:2","1:3"]
VOLUME_LIST = ["From Image","0"]
HDOS_DISKID = ["afd37dcd","c3a02220","c3a02230"] #Bytes 0-3 of disk image


VERSION = '0.6'


class gui(object):
	
	def __init__(self):
		self.config = ConfigParser.ConfigParser()
		self.rcfile = os.path.expanduser("~") + "/.h8transrc"
		if os.path.exists(self.rcfile):
			self.config.read(self.rcfile)
			self.port = self.config.get("Settings","SerialPort")
			self.baudrate = int(self.config.get("Settings", "BaudRate"))
		else:
			# lets create that config file for next time...
			cfgfile = open(self.rcfile,'w')
			self.config.add_section('Settings')
			self.config.set('Settings','SerialPort', SERIAL_PORT)
			self.config.set('Settings','BaudRate', BAUD_RATE)
			self.config.write(cfgfile)
			cfgfile.close()
			
			self.port = SERIAL_PORT
			self.baudrate = BAUD_RATE
			
		self.filename = ""
		self.volume_number = "From Image"
		self.interleave = "1:1"
		self.filepath = ""
		self.disk_image_filename = ""

		xml = gtk.Builder()
		xml.add_from_file(GLADEFILE)
		
		self.window = xml.get_object("MainWindow")
		Icombobox = xml.get_object("InterleaveCombobox")
		BRcombobox = xml.get_object("BaudrateCombobox")
		SPcomboboxentry = xml.get_object("PortComboboxentry")
		self.Volcomboboxentry = xml.get_object("VolumeComboboxentry")
		self.FTreeview = xml.get_object("FileTreeview")
		self.CTreeview = xml.get_object("ContentsTreeview")
		self.pbar = xml.get_object("Progressbar")
		self.sbar = xml.get_object("Statusbar")
		self.Sendbutton = xml.get_object("SendButton")
		self.ReceiveButton = xml.get_object("ReceiveButton")
		self.LoaderButton = xml.get_object("LoaderButton")
		
		
		xml.connect_signals(self)  

		print "h8trans.py version ", VERSION

		# Setup Combobox and Comboboxentries
		self.set_model_from_list(Icombobox,INTERLEAVE_LIST)
		Icombobox.set_active(0)
		
		self.set_model_from_list(self.Volcomboboxentry,VOLUME_LIST)
		self.Volcomboboxentry.set_active(0)

		self.set_model_from_list(BRcombobox,BAUDRATE_LIST)
		BRcombobox.set_active(BAUDRATE_LIST.index(self.baudrate))

		if self.port not in PORT_LIST:
			PORT_LIST.append(self.port)
			
		self.set_model_from_list(SPcomboboxentry,PORT_LIST)
		SPcomboboxentry.set_active(PORT_LIST.index(self.port))



		self.FListstore = gtk.ListStore(str)
		self.FTreeview.set_model(self.FListstore)
		renderer1 = gtk.CellRendererText()
		column1 =  gtk.TreeViewColumn("Disk Images", renderer1, text=0)
		column1.set_resizable(True)
		self.FTreeview.append_column(column1)

		self.CListstore = gtk.ListStore(str,str,str,str)
		self.CTreeview.set_model(self.CListstore)
		renderer2 = gtk.CellRendererText()
		column2 =  gtk.TreeViewColumn("Filename", renderer2, text=0)
		column2.set_resizable(True)
		self.CTreeview.append_column(column2)

		renderer2 = gtk.CellRendererText()
		column2 =  gtk.TreeViewColumn("Ext", renderer2, text=1)
		column2.set_resizable(True)
		self.CTreeview.append_column(column2)

		renderer2 = gtk.CellRendererText()
		column2 =  gtk.TreeViewColumn("Size", renderer2, text=2)
		column2.set_resizable(True)
		self.CTreeview.append_column(column2)

		renderer2 = gtk.CellRendererText()
		column2 =  gtk.TreeViewColumn("Date", renderer2, text=3)
		column2.set_resizable(True)
		self.CTreeview.append_column(column2)

		#dic = { "on_button1_clicked" : self.on_button1_clicked, 
		#	"on_MainWindow_destroy" : gtk.main_quit}
		#If you wanted to pass an argument, you would use a tuple like this:
    	# dic = { "on button1_clicked" : (self.button1_clicked, arg1,arg2) }

 		self.window.show_all()
  		self.pbar.hide()
  		self.sbar_cid = self.sbar.get_context_id("main")
		self.FListstore.clear()

# For testing only
#		self.filepath="/home/george/Desktop/Bootables"
#		dirList=os.listdir(self.filepath)
#		for fname in dirList:
#			if fname[-4:] == ".h8d" or fname[-4:] == ".H8D":
#				self.FListstore.append([fname])

		return

	def on_AboutMenuitem_activate(self, widget, data=None):
		dialog = gtk.AboutDialog()
		dialog.set_name('H8trans version ')
		dialog.set_version(VERSION)
		dialog.set_authors(['George Farris - farrisg@gmsys.com'])
		dialog.set_comments('H8trans is a simple program to transfer H8D floppy \
images back and forth between a Linux system and a Heathkit H8 or H89 computer\n\n \
Feature requests and bug reports welcome.')
		dialog.set_website('http://h8trans.cowlug.org')
		dialog.set_license('Distributed under the GPLv3 license.\nhttp://www.opensource.org/licenses/gpl-license.php')
		dialog.run()
		dialog.destroy()


	#Setup a ComboBox or ComboBoxEntry based on a list of strings.
	def set_model_from_list (self, cb, items):
		model = gtk.ListStore(str)
			
		for i in items:
			model.append([i])
		
		cb.set_model(model)
		
		if type(cb) == gtk.ComboBoxEntry:
			cb.set_text_column(0)
		elif type(cb) == gtk.ComboBox:
			cell = gtk.CellRendererText()
			cb.pack_start(cell, True)
			cb.add_attribute(cell, 'text', 0)
		
	def on_MainWindow_destroy(self,widget,data=None):
		gtk.main_quit()

	def on_LoaderButton_clicked(self,widget,data=None):
		print "Loader"
		try:
			sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		except:
			self.sbar.push(1,"Fatal error - Could not open serial port...")
			while gtk.events_pending():
				gtk.main_iteration()
			return

		self.ReceiveButton.set_sensitive(False)
		self.LoaderButton.set_sensitive(False)
		self.sbar.push(1,"Writing boot loader disk...")
		while gtk.events_pending():
			gtk.main_iteration()
		time.sleep(1)
		
		set_volume_number(sp, "00")
		sp.write('S')
		resp = ""
		time.sleep(4)
		resp = sp.read(1)
		if resp != 'S':
			self.sbar.push(1,"Saving the boot loader failed...")
			while gtk.events_pending():
				gtk.main_iteration()
			time.sleep(1)
			self.sbar.push(1,"Please reboot your H8 and try again...")
			while gtk.events_pending():
				gtk.main_iteration()
		else:
			self.sbar.push(1,"Finished writing boot loader...")
			while gtk.events_pending():
				gtk.main_iteration()

		self.ReceiveButton.set_sensitive(True)
		self.LoaderButton.set_sensitive(True)

		
	def on_ReceiveButton_clicked(self,widget,data=None):
		print "Receive"
		self.sbar.pop(self.sbar_cid)
		self.pbar.show()
		
		image = ""
		dialog = gtk.FileChooserDialog("Enter Disk Image filename..",
				None, gtk.FILE_CHOOSER_ACTION_SAVE, 
				(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,	gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("Disk Images")
		filter.add_pattern("*.h8d")
		dialog.add_filter(filter)


		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.filepath = dialog.get_filename()
		elif response == gtk.RESPONSE_CANCEL:
			print 'Closed, no files selected'
			dialog.destroy()
			return
		dialog.destroy()
		while gtk.events_pending():
			gtk.main_iteration()
				
		try:
			fp = open(self.filepath,"wb")
		except:
			self.sbar.push(1,"Fatal error - Could not open file...")
			while gtk.events_pending():
				gtk.main_iteration()
			return
			
		try:
			sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		except:
			self.sbar.push(1,"Fatal error - Could not open serial port...")
			while gtk.events_pending():
				gtk.main_iteration()
			return
		
		self.LoaderButton.set_sensitive(False)
		
		sp.write('R')
	
		track = 1
		new_val = 0.0
		while track <= 40:
			sp.write('R')
			buf = sp.read(2560)
			resp = sp.read(1)

			if resp == 'r':
				result = gtk.RESPONSE_CANCEL
				dlg = gtk.MessageDialog(None,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, 
					gtk.BUTTONS_CLOSE, "\nThere was an error reading the disk.\nReboot your H8 and try again...", )
				result = dlg.run()
				dlg.destroy()
				message = "Floppy read error..."
				
			elif resp == 'R':
				self.pbar.set_fraction(new_val)
				self.pbar.set_text(("track %d" % track))
				while gtk.events_pending():
					gtk.main_iteration()
				track = track + 1
				image = image + buf
				new_val = new_val + 0.025
				message = "Floppy was read without error..."

		self.pbar.hide()
		self.sbar.push(self.sbar_cid, message)
		fp.write(image)
		fp.close()	
		sp.close()
		self.LoaderButton.set_sensitive(True)
		
		
	def on_SendButton_clicked(self,widget,data=None):
		print "Send"
		self.send_image()
				
	def on_QuitButton_clicked(self,widget,data=None):
		gtk.main_quit()

	def on_QuitMenuitem_activate(self,widget,data=None):
		gtk.main_quit()
		
	def on_OpenMenuitem_activate(self,widget,data=None):
		dialog = gtk.FileChooserDialog("Select Folder...",
			None, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, 
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,	gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.filepath = dialog.get_filename()
		elif response == gtk.RESPONSE_CANCEL:
			print 'Closed, no files selected'
			dialog.destroy()
			return
		dialog.destroy()
		
		self.FListstore.clear()
		dirList=os.listdir(self.filepath)
		for fname in dirList:
			if fname[-4:] == ".h8d" or fname[-4:] == ".H8D":
				self.FListstore.append([fname])

	def on_PortComboboxentry_changed(self,widget,data=None):
		self.port = widget.get_active_text()
		print "Serial port set to " + self.port
		cfgfile = open(self.rcfile,'w')
		self.config.set('Settings','SerialPort', self.port)
		self.config.write(cfgfile)
		cfgfile.close()
		
	def on_BaudrateCombobox_changed(self,widget,data=None):
		self.baudrate = BAUDRATE_LIST[widget.get_active()]
		print ("Baud rate set to %d " % self.baudrate)
		cfgfile = open(self.rcfile,'w')
		self.config.set('Settings','BaudRate', self.baudrate)
		self.config.write(cfgfile)
		cfgfile.close()

	def on_VolumeComboboxentry_changed(self,widget,data=None):
		self.volume_number = widget.get_active_text()

	def on_InterleaveCombobox_changed(self,widget,data=None):
		self.interleave = INTERLEAVE_LIST[widget.get_active()]

	def on_FileTreeview_cursor_changed(self,widget,data=None):
		self.Sendbutton.set_sensitive(True)
		treeselection = widget.get_selection()
		model, iter = treeselection.get_selected()
		self.disk_image_filename = model.get_value(iter, 0)
		self.get_disk_directory()

	def get_disk_directory(self):
		fp = open (os.path.join(self.filepath, self.disk_image_filename), 'rb')
		
		#Get the first 4 bytes of the disk
		s = binascii.b2a_hex(fp.read(4))
		fp.seek(0,0)
		if s == HDOS_DISKID[0] or s == HDOS_DISKID[1] or s == HDOS_DISKID[2]:
			self.get_hdos_directory(fp)
		else:
			self.get_cpm_directory(fp)

	def get_hdos_directory(self,fp):
		#print "It's an HDOS disk..."
		self.sbar.push(self.sbar_cid,"Setting volume number to \"From Image\" for HDOS disks")
		self.Volcomboboxentry.set_active(0)
		self.volume_number = self.Volcomboboxentry.get_active_text()		

		self.CListstore.clear()

#seek to 0x0900
#read the 4th and 5th byte which is the sector offset of the first directory entry
#go to that location and read 512 bytes
#now in the 512 bytes there are 22 directory entries of 23 bytes in length
# 0 - 506  22 dirs
# 507 - 509  not used
# 510 - 511  next 512 byte block pointer
#
# The 22 dir entries are laid out as such:    
#	0 - 7  filename 8 bytes
#	8 - 10 extention 3 bytes
#  11 - 15 we don't use it
#  16      first group numner
#  17      last group number
#  18      last sector index
#  19 - 20 file create date
#  21 - 22 file mod date

#       |byte 20 |byte 19 |
#		0010110 0 001 11011 = 27-Jan-92
#       ------- ----- -----
#         year   mnth  day
#          |      |    |
#          |      |    +--- 27d
#          |      +--------  1d (Jan=1, Feb=2, etc.)
#          +--------------- 22d (70 + 22 = 92)

		# get the directory index and the grt then go to the dir
		fp.seek(0x903,0)
		offsethex = binascii.b2a_hex(fp.read(2))
		grt = binascii.b2a_hex(fp.read(2))
		offset = int(offsethex,16)
		fp.seek(offset, 0)

		dir_link = offsethex
		while dir_link != "0000":
			i = 22
			while i > 0:
				# read the filename and ext
				fname = fp.read(8)
				fext = fp.read(3)
				unused = fp.read(5)			
				# read the groups etc for file size			
				first_group = binascii.b2a_hex(fp.read(1))
				last_group = binascii.b2a_hex(fp.read(1))
				last_sec_index= binascii.b2a_hex(fp.read(1))
				dc = int(binascii.b2a_hex(fp.read(2)),16)
				da = fp.read(2)

				c = binascii.b2a_hex(fname[0])
				if c == "ff" or c == "00":
					i = i - 1
					continue
				if c == "fe" or c == "7f":
					break
				
				pos = fp.tell()
				fsize = self.get_hdos_filesize(fp, grt, first_group, last_group, last_sec_index)
				fp.seek(pos)

				# get the date of the file
				if dc == 0:
					cdatestr = "Null Date"
				else:
					year = ((dc & 0x00FE) >> 1) + 1970 
					month = (dc & 0x0001) + ((dc & 0xD000 >> 13))
					day = (dc & 0x1F00) >> 8
					cdate = datetime.date(year, month, day)
					cdatestr = cdate.strftime("%Y-%m-%d")
				self.CListstore.append([fname, fext, str(fsize), cdatestr])
				i = i - 1
				
			# go to next dir block
			fp.seek(int(dir_link, 16) + 510)
			dir_link = binascii.b2a_hex(fp.read(2))
			fp.seek(int(dir_link,16))


		
	def get_hdos_filesize(self, fp1, grt, fg, lg, lsidx):
		fs = 2
		#Go to the grt + the first group
		fp = fp1
		fp.seek(int(grt,16) + int(fg,16))
		ng = binascii.b2a_hex(fp.read(1))
		while ng != "00":
			fp.seek(int(grt,16) + int(ng,16))
			ng = binascii.b2a_hex(fp.read(1))
			fs = fs + 2

		if lsidx == "01":
			fs = fs - 1
		return fs
		
		
	def get_cpm_directory(self,fp):
		#print "It's an CP/M disk..."
		self.sbar.push(self.sbar_cid,"Setting volume number to \"0\" for CP/M disks")
		self.Volcomboboxentry.set_active(1)
		self.volume_number = self.Volcomboboxentry.get_active_text()		

		# Start at offset 0x1E00		
		#read dir entry until flag == 0xE5 and filenma[0] = 0xE5
		#jump to offset 0x2200
		#read dir entry until flag == 0xE5 and filenma[0] = 0xE5		
		#jump to offset 0x2600
		#read dir entry until flag == 0xE5 and filenma[0] = 0xE5		
		#
		# disk entry is 32 bytes long and looks like so:
		# byte 0  flag		# byte 1-8 filename		# byte 9-11 extention
		# byte 12 extent	# byte 13-14 unused		# byte 15 sector count
		# byte 16-32 allocation map
		self.CListstore.clear()
				
		fp.seek(0x1e00, 0)
		rec = fp.read(32)
		while binascii.b2a_hex(rec[0]) != "e5" and binascii.b2a_hex(rec[1]) != "e5":
			fname = rec[1:9]
			fext = rec[9:12]
			fsize = self.cpm_file_size(rec[15])
			self.CListstore.append([fname, fext, fsize, ""])
			rec = fp.read(32)
		fp.seek(0x2200, 0)
		rec = fp.read(32)
		while binascii.b2a_hex(rec[0]) != "e5" and binascii.b2a_hex(rec[1]) != "e5":
			fname = rec[1:9]
			fext = rec[9:12]
			fsize = self.cpm_file_size(rec[15])
			self.CListstore.append([fname, fext, fsize, ""])
			rec = fp.read(32)
		fp.seek(0x2600,0)
		rec = fp.read(32)
		while binascii.b2a_hex(rec[0]) != "e5" and binascii.b2a_hex(rec[1]) != "e5":
			fname = rec[1:9]
			fext = rec[9:12]
			fsize = self.cpm_file_size(rec[15])
			self.CListstore.append([fname, fext, fsize, ""])
			rec = fp.read(32)

	def cpm_file_size(self,sectors):
		sec = ord(sectors)  #ord =  char to int
		if sec < 8:
			return "1k"
		else:
			tmp = int((sec * 128) / 1024)
			if (sec * 128) % 1024 > 0:
				tmp = tmp + 1
			return "%sk" % tmp

	def send_image(self):
		self.sbar.pop(self.sbar_cid)
		self.pbar.show()
#		self.sbar.push(1," ")

		try:
			fp = open (os.path.join(self.filepath, self.disk_image_filename), 'rb')
		except:
			self.sbar.push(1,"Fatal error - Could not open file...")
			while gtk.events_pending():
				gtk.main_iteration()
			return
			
		try:
			sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		except:
			self.sbar.push(1,"Fatal error - Could not open serial port...")
			while gtk.events_pending():
				gtk.main_iteration()
			return
	
		self.LoaderButton.sensitive(False)
		self.ReceiveButton.sensitive(False)

		if typeofdisk == "cpm":
			set_volume_number(sp,"00")
		else:
			fp.seek(0x900,0)
			vol = binascii.b2a_hex(fp.read(1))
			fp.seek(0,0)
			set_volume_number(sp,vol)
			
		set_interleave(sp,"00")

		sp.write('W')
		sp.read(1)

		track = 1
		new_val = 0.0
	
		while track <= 40:
			sp.write('W')
			buf = binascii.b2a_hex(fp.read(2560))
			sp.write(binascii.a2b_hex(buf))
			time.sleep(4)			

			resp = sp.read(1)			#'W' if ok
			if resp != 'W':
				self.sbar.push(1,"Fatal error - Floppy track write failed...")
				while gtk.events_pending():
					gtk.main_iteration()
			else:
				self.pbar.set_fraction(new_val)
				self.pbar.set_text(("track %d" % track))
				while gtk.events_pending():
					gtk.main_iteration()
				track = track + 1
				new_val = new_val + 0.025
				message = "Floppy was written without error..."

		fp.close()	
		sp.close()
		self.Sendbutton.set_sensitive(False)
		self.LoaderButton.sensitive(True)
		self.ReceiveButton.sensitive(True)
		self.FTreeview.unselect_all()

	# Run main application window
	def main(self):
		self.window.show()
		gtk.main()


#=============================================================================================
#
#  Command line stuff.
#
#=============================================================================================
		
		
'''		
; S - Save this image loader to disk as a stand alone boot
;      ( not HDOS ). The disk must be originally formatted
;      with V = 0. It returns a S when complete.'''
def save_image_loader():
	try:
		sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		print "%s%s%s%s%s" % ("Serial port open [",SERIAL_PORT," ] [",BAUD_RATE,"]...")
	except:
		print "Fatal error - Could not open serial port...\n"
		print "Exiting..."
		sys.exit(1)

	set_volume_number(sp, "00")
	sp.write('S')
	resp = ""
	time.sleep(4)
	resp = sp.read(1)
	if resp != 'S':
		print "Saving the bootloader failed..."
		print "Please reboot your H8 and try again..."
		sys.exit(1)

		
'''
; I - Followed by a number 0,1,2 corresponding to a sector
;      interleaving of 1:1, 1:2, or 1:3. Other numbers will
;      cause incorrect formatting during writes. This has
;      effect on the W and S commands only.'''
def set_interleave(sp,factor):
	sp.write('I')
	#sp.write(binascii.a2b_hex(vol))
	sp.write(binascii.a2b_hex(factor))
	resp = ""
	resp = sp.read(1)
	if resp != 'I':
		print "Setting the interleave failed..."
		print "Please reboot your H8 and try again..."
		sys.exit(1)

		
'''
; V - Sets the volume number for the various operations.
;      Check HDOS docs for useage of the Volume number.
;      It must receive the volume number as a non-ascii
;      byte as the next byte after the the V command. It
;      returns a V when complete.'''
def set_volume_number(sp,vol):
	sp.write('V')
	sp.write(binascii.a2b_hex(vol))
	resp = ""
	resp = sp.read(1)
	if resp != 'V':
		print "Setting the volume number failed..."
		print "Please reboot your H8 and try again..."
		sys.exit(1)

'''
; C - Read the disk and returns the volume number if it is
;      an HDOS disk. If it is another type it would be an
;      indeterminate value.'''
def get_volume_number(sp):
	sp.write('C')
	resp = -1
	resp = sp.read(1)
	if resp == -1:
		print "Getting the volume number failed..."
		print "Please reboot your H8 and try again..."
		sys.exit(1)
	return(resp)
	
	
'''
; R - Reads a disk image, using  V value. Each buffer will
;     begin transfer when a 'R' is received. Buffer size
;     is the same as used for W command. A character, other
;     than r will abort command. Buffer data is transfered
;     as 8 bit values. Response handshake is 'r' if error.'''
def cmdline_read_floppy(typeofdisk,filename):
	print "\nRead disk image from H8...\n"
	
	image = ""
	myfile = open(filename,"wb")
	track = 40

	try:
		sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		print "%s%s%s%s%s" % ("Serial port open [",SERIAL_PORT," ] [",BAUD_RATE,"]...")
	except:
		print "Fatal error - Could not open serial port...\n"
		print "Exiting..."
		sys.exit(1)

	sp.write('R')
	print "\nTracks:"
	
	while track:
		sp.write('R')
		buf = sp.read(2560)
		resp = sp.read(1)			#'R' if ok, 'r' is error

		if resp == 'r':
			print "\nThere was an error reading the disk, please try again..."
			print "Please reboot your H8 and try again..."
			sys.exit(1)
		elif resp == 'R':
			if track in [40,30,20,10]:
				sys.stdout.write("%d" % track)
				sys.stdout.flush()
			else:
				sys.stdout.write(".")
				sys.stdout.flush()
			track = track - 1
			image = image + buf
			
	print "0\n\nFinished..."
	myfile.write(image)
	myfile.close()	
	sp.close()
	sys.exit()



'''
; W - Write image to disk. When each buffer is loaded
;      it replies with W. Each buffer must start with 'W'
;      or it will abort to command. Buffer size must match.
;      It expects the V value to match the disk in the drive.
;      Buffer data is transfered as 8 bit value.'''
def cmdline_write_floppy(typeofdisk, filename):
	print "\nWrite disk image to H8...\n"
	
	image = ""
	myfile = open(filename,"rb")

	track = 40

	try:
		sp = serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1)
		print "%s%s%s%s%s" % ("Serial port open [",SERIAL_PORT," ] [",BAUD_RATE,"]...")
	except:
		print "Fatal error - Could not open serial port...\n"
		print "Exiting..."
		sys.exit(1)

	if typeofdisk == "cpm":
		set_volume_number(sp,"00")
	else:
		myfile.seek(0x900,0)
		vol = binascii.b2a_hex(myfile.read(1))
		myfile.seek(0,0)
		set_volume_number(sp,vol)

	set_interleave(sp,"00")

	sp.write('W')
	sp.read(1)
	print "\nTracks:"
	
	while track:
		sp.write('W')
		buf = binascii.b2a_hex(myfile.read(2560))
		sp.write(binascii.a2b_hex(buf))
		time.sleep(4)			

		resp = sp.read(1)			#'W' if ok
		if resp != 'W':
			print "\nThere was an error wrinting the disk, please try again..."
			print "Please reboot your H8 and try again..."
			sys.exit(1)
		else:
			if track in [40,30,20,10]:
				sys.stdout.write("%d" % track)
				sys.stdout.flush()
			else:
				sys.stdout.write(".")
				sys.stdout.flush()
		track = track - 1
				
	print "0\n\nFinished..."
	print "I suggest you reboot your H8 if you want to read floppies reliably..."
	myfile.close()	
	sp.close()
	sys.exit()

def usage():
	print "\n"
	print "========================================================================"
	print "h8trans.py - Copyright George Farris, 2011. - GPLv3"
	print "========================================================================\n"

	print "h8trans.py is used to read, write and save the boot loader on an H8"
	print "or H89 computer system.  It has two modes that it will run in."
	print "\nMode one is graphical and all options are set from the GUI."
	print "Mode two is via the command line and is used as follows.\n"
	print "To read a floppy image on the command line:"
	print "  h8trans.py -r -f filename"
	print "To write a floppy image on the command line:"
	print "  h8trans.py -w -t [cpm|hdos] -f filename"
	print "\nTo save the bootloader run:"
	print "  h8trans.py -s"
	print "\n"



if __name__ == "__main__":
	
	myfilename = ""
	typeofdisk = ""
			
	try:                                
		opts, args = getopt.getopt(sys.argv[1:], "ohf:rwst:", ["volume","help","file","read","write","save","type"])
	except getopt.GetoptError:
		usage()
		sys.exit(0)
	
	cmd = "gui"
	for opt, arg in opts:
		if opt in ("-h", "--help"):
			usage()
			sys.exit(0)                  
		elif opt in ("-o", "--volume"):
			print "option o"
		elif opt in ("-f", "--file"):
			myfilename = arg
			print "Filename " + myfilename
		elif opt in ("-t", "--type"):
			typeofdisk = arg
			print "Type of disk " + typeofdisk
		elif opt in ("-w", "--write"):
			cmd = 'write'
		elif opt in ("-r", "--read"):
			cmd = 'read'
		elif opt in ("-s", "--save"):
			cmd = 'save'			
		
	if cmd == 'read':
		cmdline_read_floppy(typeofdisk, myfilename)
		sys.exit(1)
	elif cmd == 'write':
		if typeofdisk == "cpm" or typeofdisk == "hdos":
			cmdline_write_floppy(typeofdisk,myfilename)
		else:
			print "\nYou must set a disk type with the -t switch.  The type can be"
			print "cpm or hdos. Example -t cpm or -t hdos\n"
			sys.exit(1)
	elif cmd == 'save':
		save_image_loader()
	elif cmd == 'gui':
		hwg = gui()
		hwg.main()
	elif cmd == "":
		print "You must choose -r, -w -s or -g to read, write, save or run in GUI mode..."
		sys.exit(1)
	
	#
	print "Goodbye..."
